var $                = require("jquery"),
    packir           = require("packir"),
    _                = require("lodash"),
    parallel         = require("../utils/parallel"),
    BaseStrategy     = packir.BaseStrategy,
    StackCompressor  = packir.StackCompressor,
    Rule             = packir.Rule,
    NonrecurringRule = packir.NonrecurringRule,
    RecurringRule    = packir.RecurringRule,
    Area             = packir.Area,
    _old;

_old = $.fn.packir;

$.fn.packir = function(opts) {
    var options, strategy, compressor, done;

    options = _.defaults({}, opts, {
        strategy: "base",
        compressor: "stack",
        rules: [
            "nonrecurring",
            "recurring"
        ],
        data: {
            weight: "weight"
        },
        callback: _.noop
    });

    done = options.callback;

    switch (options.strategy) {
        default: {
            strategy = new BaseStrategy();
        }
    }

    switch (options.compressor) {
        default: {
            compressor = new StackCompressor();
        }
    }

    strategy.injectCompressor(compressor);

    _.forEach(options.rules, function(def) {
        var rule, options, r;

        if (_.isArray(def)) {
            rule = def[0];
            options = def[1];
        } else {
            rule = def;
        }

        switch (rule) {
            case "nonrecurring": {
                r = new NonrecurringRule(options);
                break;
            }

            case "recurring": {
                r = new RecurringRule(options);
                break;
            }

            default: {
                if (rule instanceof Rule) {
                    return rule;
                }

                throw new Error('Unknown rule: "' + rule + '"');
            }
        }

        strategy.addRule(r);
    });

    parallel(
        this.map(function() {
            var $el = $(this);

            return function(next) {
                var area, blocks;

                area = new Area($el.width(), $el.height());
                area.injectStrategy(strategy);

                blocks = $el.children().map(function(index) {
                    return {
                        width:  $(this).width(),
                        height: $(this).height(),
                        weight: $(this).data(options.data.weight) || index,
                        $el:    $(this)
                    };
                });

                area.fill(blocks, function(err, grid) {
                    var $grid;

                    if (err) {
                        next(err);
                        return;
                    }

                    $grid = $("<div/>");

                    _.forEach(grid.rows, function(row) {
                        var $row, stackLeft;

                        $row = $("<div/>").css({
                            width: row.maxWidth * grid.leftStep + 'px',
                            height: row.maxHeight * grid.topStep + 'px'
                        });

                        stackLeft = 0;

                        _.forEach(row.stacks, function(stack) {
                            var $stack, width, height;

                            width = stack.maxWidth * grid.leftStep;
                            height = stack.maxHeight * grid.topStep;

                            $stack = $("<div/>").css({
                                position: "absolute",
                                top: 0,
                                left: stackLeft + 'px',
                                width: width + 'px',
                                height: height + 'px'
                            });

                            stackLeft+= width;

                            _.forEach(stack.fields, function(field) {
                                var $field;

                                $field = $("<div/>").css({
                                    position: "absolute",
                                    left: field.pointer.left * grid.leftStep + 'px',
                                    top: field.pointer.top * grid.topStep + 'px',
                                    width: (width = field.width * grid.leftStep) + 'px',
                                    height: (height = field.height * grid.topStep) + 'px'
                                });

                                $field.append(field.data.$el);
                                $stack.append($field);
                            });

                            $row.append($stack);
                        });

                        $grid.append($row);
                    });

                    $el.append($grid);

                    next();
                });
            };
        }),
        done
    );
};

$.fn.packir.noConflict = function() {
    $.fn.packir = _old;
};

